# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - LocationSupport library (derived from EventAggregatorSupport)

    @copyright: 2011, 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

import operator
import re

location_normalised_regexp = re.compile(
    ur"(?:\d+\w*\s+)?"          # preceding postcode (optional)
    ur"(?P<location>"           # start of group of interest
    ur"\w[\w\s-]+?"             # area or town
    ur"(?:,(?:\s*[\w-]+)+)?"    # country (optional)
    ur")$", re.UNICODE)

# Utility functions.

def sign(x):
    if x < 0:
        return -1
    else:
        return 1

# Location-related functions.

class Reference:

    "A map reference."

    def __init__(self, degrees, minutes=0, seconds=0):
        self.degrees = degrees
        self.minutes = minutes
        self.seconds = seconds

    def __repr__(self):
        return "Reference(%d, %d, %f)" % (self.degrees, self.minutes, self.seconds)

    def __str__(self):
        return "%d:%d:%f" % (self.degrees, self.minutes, self.seconds)

    def __add__(self, other):
        if not isinstance(other, Reference):
            return NotImplemented
        else:
            s = sign(self.degrees)
            o = sign(other.degrees)
            carry, seconds = adc(s * self.seconds, o * other.seconds)
            carry, minutes = adc(s * self.minutes, o * other.minutes + carry)
            return Reference(self.degrees + other.degrees + carry, minutes, seconds)

    def __sub__(self, other):
        if not isinstance(other, Reference):
            return NotImplemented
        else:
            return self.__add__(Reference(-other.degrees, other.minutes, other.seconds))

    def _compare(self, op, other):
        if not isinstance(other, Reference):
            return NotImplemented
        else:
            return op(self.to_degrees(), other.to_degrees())

    def __eq__(self, other):
        return self._compare(operator.eq, other)

    def __ne__(self, other):
        return self._compare(operator.ne, other)

    def __lt__(self, other):
        return self._compare(operator.lt, other)

    def __le__(self, other):
        return self._compare(operator.le, other)

    def __gt__(self, other):
        return self._compare(operator.gt, other)

    def __ge__(self, other):
        return self._compare(operator.ge, other)

    def to_degrees(self):
        return sign(self.degrees) * (abs(self.degrees) + self.minutes / 60.0 + self.seconds / 3600.0)

    def to_pixels(self, scale):
        return self.to_degrees() * scale

def adc(x, y):
    result = x + y
    return divmod(result, 60)

def getPositionForReference(latitude, longitude, map_y, map_x, map_x_scale, map_y_scale):
    return (longitude - map_x).to_pixels(map_x_scale), (latitude - map_y).to_pixels(map_y_scale)

def getPositionForCentrePoint(position, map_x_scale, map_y_scale):
    x, y = position
    return x - map_x_scale / 2.0, y - map_y_scale / 2.0

def getMapReference(value):

    "Return a map reference by parsing the given 'value'."

    if value.find(":") != -1:
        return getMapReferenceFromDMS(value)
    else:
        return getMapReferenceFromDecimal(value)

def getMapReferenceFromDMS(value):

    """
    Return a map reference by parsing the given 'value' expressed as degrees,
    minutes, seconds.
    """

    values = value.split(":")
    values = map(int, values[:2]) + map(float, values[2:3])
    return Reference(*values)

def getMapReferenceFromDecimal(value):

    "Return a map reference by parsing the given 'value' in decimal degrees."

    value = float(value)
    degrees, remainder = divmod(abs(value * 3600), 3600)
    minutes, seconds = divmod(remainder, 60) 
    return Reference(sign(value) * degrees, minutes, seconds)

# User interface functions.

def getNormalisedLocation(location):

    """
    Attempt to return a normalised 'location' of the form "<town>, <country>" or
    "<town>".
    """

    match = location_normalised_regexp.search(location)
    if match:
        return match.group("location")
    else:
        return None

# vim: tabstop=4 expandtab shiftwidth=4
